@@ -34,6 +34,7 @@
 
 #include "mmcli.h"
 #include "mmcli-common.h"
+#include "mmcli-output.h"
 
 #define PROGRAM_NAME    "mmcli"
 #define PROGRAM_VERSION PACKAGE_VERSION
@@ -43,12 +44,22 @@ static GMainLoop *loop;
 static GCancellable *cancellable;
 
 /* Context */
+static gboolean output_keyvalue_flag;
+static gboolean output_json_flag;
 static gboolean verbose_flag;
 static gboolean version_flag;
 static gboolean async_flag;
 static gint timeout = 30; /* by default, use 30s for all operations */
 
 static GOptionEntry main_entries[] = {
+    { "output-keyvalue", 'K', 0, G_OPTION_ARG_NONE, &output_keyvalue_flag,
+      "Run action with machine-friendly key-value output",
+      NULL
+    },
+    { "output-json", 'J', 0, G_OPTION_ARG_NONE, &output_json_flag,
+      "Run action with machine-friendly json output",
+      NULL
+    },
     { "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose_flag,
       "Run action with verbose logs",
       NULL
@@ -110,7 +121,6 @@ log_handler (const gchar *log_domain,
         break;
 
     case G_LOG_LEVEL_CRITICAL:
-    case G_LOG_FLAG_FATAL:
     case G_LOG_LEVEL_ERROR:
         log_level_str = "-Error **";
         break;
@@ -119,9 +129,16 @@ log_handler (const gchar *log_domain,
         log_level_str = "[Debug]";
         break;
 
-    default:
+    case G_LOG_LEVEL_MESSAGE:
+    case G_LOG_LEVEL_INFO:
         log_level_str = "";
         break;
+
+    case G_LOG_FLAG_FATAL:
+    case G_LOG_LEVEL_MASK:
+    case G_LOG_FLAG_RECURSION:
+    default:
+        g_assert_not_reached ();
     }
 
     g_print ("[%s] %s %s\n", time_str, log_level_str, message);
@@ -130,9 +147,8 @@ log_handler (const gchar *log_domain,
 static void
 print_version_and_exit (void)
 {
-    g_print ("\n"
-             PROGRAM_NAME " " PROGRAM_VERSION "\n"
-             "Copyright (2011 - 2016) Aleksander Morgado\n"
+    g_print (PROGRAM_NAME " " PROGRAM_VERSION "\n"
+             "Copyright (2011 - 2021) Aleksander Morgado\n"
              "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\n"
              "This is free software: you are free to change and redistribute it.\n"
              "There is NO WARRANTY, to the extent permitted by law.\n"
@@ -230,6 +246,28 @@ main (gint argc, gchar **argv)
     if (verbose_flag)
         g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_MASK, log_handler, NULL);
 
+    /* Setup output */
+    if (output_keyvalue_flag && output_json_flag) {
+        g_printerr ("error: only one output type supported at the same time\n");
+        exit (EXIT_FAILURE);
+    }
+    if (output_keyvalue_flag) {
+        if (verbose_flag) {
+            g_printerr ("error: cannot set verbose output in keyvalue output type\n");
+            exit (EXIT_FAILURE);
+        }
+        mmcli_output_set (MMC_OUTPUT_TYPE_KEYVALUE);
+    }
+    else if (output_json_flag) {
+        if (verbose_flag) {
+            g_printerr ("error: cannot set verbose output in JSON output type\n");
+            exit (EXIT_FAILURE);
+        }
+        mmcli_output_set (MMC_OUTPUT_TYPE_JSON);
+    } else {
+        mmcli_output_set (MMC_OUTPUT_TYPE_HUMAN);
+    }
+
     /* Setup signals */
     signal (SIGINT, signals_handler);
     signal (SIGHUP, signals_handler);
