@@ -38,10 +38,12 @@ G_DEFINE_TYPE (MMBearerProperties, mm_bearer_properties, G_TYPE_OBJECT);
 #define PROPERTY_USER            "user"
 #define PROPERTY_PASSWORD        "password"
 #define PROPERTY_IP_TYPE         "ip-type"
-#define PROPERTY_NUMBER          "number"
 #define PROPERTY_ALLOW_ROAMING   "allow-roaming"
 #define PROPERTY_RM_PROTOCOL     "rm-protocol"
 
+/* no longer used properties */
+#define DEPRECATED_PROPERTY_NUMBER "number"
+
 struct _MMBearerPropertiesPrivate {
     /* APN */
     gchar *apn;
@@ -49,8 +51,6 @@ struct _MMBearerPropertiesPrivate {
     MMBearerIpFamily ip_type;
     /* Allowed auth */
     MMBearerAllowedAuth allowed_auth;
-    /* Number */
-    gchar *number;
     /* User */
     gchar *user;
     /* Password */
@@ -70,6 +70,8 @@ struct _MMBearerPropertiesPrivate {
  * @apn: Name of the access point.
  *
  * Sets the name of the access point to use when connecting.
+ *
+ * Since: 1.0
  */
 void
 mm_bearer_properties_set_apn (MMBearerProperties *self,
@@ -87,7 +89,10 @@ mm_bearer_properties_set_apn (MMBearerProperties *self,
  *
  * Gets the name of the access point to use when connecting.
  *
- * Returns: (transfer none): the access point, or #NULL if not set. Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): the access point, or #NULL if not set. Do not free
+ * the returned value, it is owned by @self.
+ *
+ * Since: 1.0
  */
 const gchar *
 mm_bearer_properties_get_apn (MMBearerProperties *self)
@@ -102,9 +107,13 @@ mm_bearer_properties_get_apn (MMBearerProperties *self)
 /**
  * mm_bearer_properties_set_allowed_auth:
  * @self: a #MMBearerProperties.
- * @allowed_auth: a bitmask of #MMBearerAllowedAuth values. %MM_BEARER_ALLOWED_AUTH_UNKNOWN may be given to request the modem-default method.
+ * @allowed_auth: a bitmask of #MMBearerAllowedAuth values.
+ *  %MM_BEARER_ALLOWED_AUTH_UNKNOWN may be given to request the modem-default
+ *  method.
  *
  * Sets the authentication method to use.
+ *
+ * Since: 1.0
  */
 void
 mm_bearer_properties_set_allowed_auth (MMBearerProperties *self,
@@ -121,7 +130,10 @@ mm_bearer_properties_set_allowed_auth (MMBearerProperties *self,
  *
  * Gets the authentication methods allowed in the connection.
  *
- * Returns: a bitmask of #MMBearerAllowedAuth values, or %MM_BEARER_ALLOWED_AUTH_UNKNOWN to request the modem-default method.
+ * Returns: a bitmask of #MMBearerAllowedAuth values, or
+ * %MM_BEARER_ALLOWED_AUTH_UNKNOWN to request the modem-default method.
+ *
+ * Since: 1.0
  */
 MMBearerAllowedAuth
 mm_bearer_properties_get_allowed_auth (MMBearerProperties *self)
@@ -139,6 +151,8 @@ mm_bearer_properties_get_allowed_auth (MMBearerProperties *self)
  * @user: the username
  *
  * Sets the username used to authenticate with the access point.
+ *
+ * Since: 1.0
  */
 void
 mm_bearer_properties_set_user (MMBearerProperties *self,
@@ -156,7 +170,10 @@ mm_bearer_properties_set_user (MMBearerProperties *self,
  *
  * Gets the username used to authenticate with the access point.
  *
- * Returns: (transfer none): the username, or #NULL if not set. Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): the username, or #NULL if not set. Do not free the
+ * returned value, it is owned by @self.
+ *
+ * Since: 1.0
  */
 const gchar *
 mm_bearer_properties_get_user (MMBearerProperties *self)
@@ -174,6 +191,8 @@ mm_bearer_properties_get_user (MMBearerProperties *self)
  * @password: the password
  *
  * Sets the password used to authenticate with the access point.
+ *
+ * Since: 1.0
  */
 void
 mm_bearer_properties_set_password (MMBearerProperties *self,
@@ -191,7 +210,10 @@ mm_bearer_properties_set_password (MMBearerProperties *self,
  *
  * Gets the password used to authenticate with the access point.
  *
- * Returns: (transfer none): the password, or #NULL if not set. Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): the password, or #NULL if not set. Do not free
+ * the returned value, it is owned by @self.
+ *
+ * Since: 1.0
  */
 const gchar *
 mm_bearer_properties_get_password (MMBearerProperties *self)
@@ -209,6 +231,8 @@ mm_bearer_properties_get_password (MMBearerProperties *self)
  * @ip_type: a #MMBearerIpFamily.
  *
  * Sets the IP type to use.
+ *
+ * Since: 1.0
  */
 void
 mm_bearer_properties_set_ip_type (MMBearerProperties *self,
@@ -226,11 +250,13 @@ mm_bearer_properties_set_ip_type (MMBearerProperties *self,
  * Sets the IP type to use.
  *
  * Returns: a #MMBearerIpFamily.
+ *
+ * Since: 1.0
  */
 MMBearerIpFamily
 mm_bearer_properties_get_ip_type (MMBearerProperties *self)
 {
-    g_return_val_if_fail (MM_IS_BEARER_PROPERTIES (self), MM_BEARER_IP_FAMILY_UNKNOWN);
+    g_return_val_if_fail (MM_IS_BEARER_PROPERTIES (self), MM_BEARER_IP_FAMILY_NONE);
 
     return self->priv->ip_type;
 }
@@ -244,6 +270,8 @@ mm_bearer_properties_get_ip_type (MMBearerProperties *self)
  *
  * Sets the flag to indicate whether roaming is allowed or not in the
  * connection.
+ *
+ * Since: 1.0
  */
 void
 mm_bearer_properties_set_allow_roaming (MMBearerProperties *self,
@@ -261,7 +289,9 @@ mm_bearer_properties_set_allow_roaming (MMBearerProperties *self,
  *
  * Checks whether roaming is allowed in the connection.
  *
- * Returns: %TRUE if roaming is allowed, %FALSE otherwise..
+ * Returns: %TRUE if roaming is allowed, %FALSE otherwise.
+ *
+ * Since: 1.0
  */
 gboolean
 mm_bearer_properties_get_allow_roaming (MMBearerProperties *self)
@@ -273,12 +303,18 @@ mm_bearer_properties_get_allow_roaming (MMBearerProperties *self)
 
 /*****************************************************************************/
 
+#ifndef MM_DISABLE_DEPRECATED
+
 /**
  * mm_bearer_properties_set_number:
  * @self: a #MMBearerProperties.
  * @number: the number.
  *
  * Sets the number to use when performing the connection.
+ *
+ * Since: 1.0
+ * Deprecated: 1.10.0. The number setting is not used anywhere, and therefore
+ * it doesn't make sense to expose it in the ModemManager interface.
  */
 void
 mm_bearer_properties_set_number (MMBearerProperties *self,
@@ -286,8 +322,7 @@ mm_bearer_properties_set_number (MMBearerProperties *self,
 {
     g_return_if_fail (MM_IS_BEARER_PROPERTIES (self));
 
-    g_free (self->priv->number);
-    self->priv->number = g_strdup (number);
+    /* NO-OP */
 }
 
 /**
@@ -296,24 +331,34 @@ mm_bearer_properties_set_number (MMBearerProperties *self,
  *
  * Gets the number to use when performing the connection.
  *
- * Returns: (transfer none): the number, or #NULL if not set. Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): the number, or #NULL if not set. Do not free the
+ * returned value, it is owned by @self.
+ *
+ * Since: 1.0
+ * Deprecated: 1.10.0. The number setting is not used anywhere, and therefore
+ * it doesn't make sense to expose it in the ModemManager interface.
  */
 const gchar *
 mm_bearer_properties_get_number (MMBearerProperties *self)
 {
     g_return_val_if_fail (MM_IS_BEARER_PROPERTIES (self), NULL);
 
-    return self->priv->number;
+    /* NO-OP */
+    return NULL;
 }
 
+#endif /* MM_DISABLE_DEPRECATED */
+
 /*****************************************************************************/
 
 /**
- * mm_simple_connect_properties_set_rm_protocol:
+ * mm_bearer_properties_set_rm_protocol:
  * @self: a #MMBearerProperties.
  * @protocol: a #MMModemCdmaRmProtocol.
  *
  * Sets the RM protocol to use in the CDMA connection.
+ *
+ * Since: 1.0
  */
 void
 mm_bearer_properties_set_rm_protocol (MMBearerProperties *self,
@@ -331,6 +376,8 @@ mm_bearer_properties_set_rm_protocol (MMBearerProperties *self,
  * Gets the RM protocol requested to use in the CDMA connection.
  *
  * Returns: a #MMModemCdmaRmProtocol.
+ *
+ * Since: 1.0
  */
 MMModemCdmaRmProtocol
 mm_bearer_properties_get_rm_protocol (MMBearerProperties *self)
@@ -342,6 +389,9 @@ mm_bearer_properties_get_rm_protocol (MMBearerProperties *self)
 
 /*****************************************************************************/
 
+/**
+ * mm_bearer_properties_get_dictionary: (skip)
+ */
 GVariant *
 mm_bearer_properties_get_dictionary (MMBearerProperties *self)
 {
@@ -379,18 +429,12 @@ mm_bearer_properties_get_dictionary (MMBearerProperties *self)
                                PROPERTY_PASSWORD,
                                g_variant_new_string (self->priv->password));
 
-    if (self->priv->ip_type != MM_BEARER_IP_FAMILY_UNKNOWN)
+    if (self->priv->ip_type != MM_BEARER_IP_FAMILY_NONE)
         g_variant_builder_add (&builder,
                                "{sv}",
                                PROPERTY_IP_TYPE,
                                g_variant_new_uint32 (self->priv->ip_type));
 
-    if (self->priv->number)
-        g_variant_builder_add (&builder,
-                               "{sv}",
-                               PROPERTY_NUMBER,
-                               g_variant_new_string (self->priv->number));
-
     if (self->priv->allow_roaming_set)
         g_variant_builder_add (&builder,
                                "{sv}",
@@ -408,6 +452,9 @@ mm_bearer_properties_get_dictionary (MMBearerProperties *self)
 
 /*****************************************************************************/
 
+/**
+ * mm_bearer_properties_consume_string: (skip)
+ */
 gboolean
 mm_bearer_properties_consume_string (MMBearerProperties *self,
                                      const gchar *key,
@@ -452,9 +499,7 @@ mm_bearer_properties_consume_string (MMBearerProperties *self,
             return FALSE;
         }
         mm_bearer_properties_set_allow_roaming (self, allow_roaming);
-    } else if (g_str_equal (key, PROPERTY_NUMBER))
-        mm_bearer_properties_set_number (self, value);
-    else if (g_str_equal (key, PROPERTY_RM_PROTOCOL)) {
+    } else if (g_str_equal (key, PROPERTY_RM_PROTOCOL)) {
         GError *inner_error = NULL;
         MMModemCdmaRmProtocol protocol;
 
@@ -464,11 +509,13 @@ mm_bearer_properties_consume_string (MMBearerProperties *self,
             return FALSE;
         }
         mm_bearer_properties_set_rm_protocol (self, protocol);
+    } else if (g_str_equal (key, DEPRECATED_PROPERTY_NUMBER)) {
+        /* NO-OP */
     } else {
         g_set_error (error,
                      MM_CORE_ERROR,
-                     MM_CORE_ERROR_INVALID_ARGS,
-                     "Invalid properties string, unexpected key '%s'",
+                     MM_CORE_ERROR_UNSUPPORTED,
+                     "Invalid properties string, unsupported key '%s'",
                      key);
         return FALSE;
     }
@@ -492,6 +539,9 @@ key_value_foreach (const gchar *key,
                                                 &ctx->error);
 }
 
+/**
+ * mm_bearer_properties_new_from_string: (skip)
+ */
 MMBearerProperties *
 mm_bearer_properties_new_from_string (const gchar *str,
                                       GError **error)
@@ -517,6 +567,9 @@ mm_bearer_properties_new_from_string (const gchar *str,
 
 /*****************************************************************************/
 
+/**
+ * mm_bearer_properties_consume_variant: (skip)
+ */
 gboolean
 mm_bearer_properties_consume_variant (MMBearerProperties *properties,
                                       const gchar *key,
@@ -545,15 +598,17 @@ mm_bearer_properties_consume_variant (MMBearerProperties *properties,
         mm_bearer_properties_set_ip_type (
             properties,
             g_variant_get_uint32 (value));
-    else if (g_str_equal (key, PROPERTY_NUMBER))
-        mm_bearer_properties_set_number (
-            properties,
-            g_variant_get_string (value, NULL));
     else if (g_str_equal (key, PROPERTY_ALLOW_ROAMING))
         mm_bearer_properties_set_allow_roaming (
             properties,
             g_variant_get_boolean (value));
-    else {
+    else if (g_str_equal (key, PROPERTY_RM_PROTOCOL))
+        mm_bearer_properties_set_rm_protocol (
+            properties,
+            g_variant_get_uint32 (value));
+    else if (g_str_equal (key, DEPRECATED_PROPERTY_NUMBER)) {
+        /* NO-OP */
+    } else {
         /* Set error */
         g_set_error (error,
                      MM_CORE_ERROR,
@@ -566,6 +621,9 @@ mm_bearer_properties_consume_variant (MMBearerProperties *properties,
     return TRUE;
 }
 
+/**
+ * mm_bearer_properties_new_from_dictionary: (skip)
+ */
 MMBearerProperties *
 mm_bearer_properties_new_from_dictionary (GVariant *dictionary,
                                           GError **error)
@@ -613,38 +671,102 @@ mm_bearer_properties_new_from_dictionary (GVariant *dictionary,
 
 /*****************************************************************************/
 
-MMBearerProperties *
-mm_bearer_properties_dup (MMBearerProperties *orig)
+static gboolean
+cmp_str (const gchar                *a,
+         const gchar                *b,
+         MMBearerPropertiesCmpFlags  flags)
 {
-    GVariant *dict;
-    MMBearerProperties *copy;
-    GError *error = NULL;
-
-    g_return_val_if_fail (MM_IS_BEARER_PROPERTIES (orig), NULL);
-
-    dict = mm_bearer_properties_get_dictionary (orig);
-    copy = mm_bearer_properties_new_from_dictionary (dict, &error);
-    g_assert_no_error (error);
-    g_variant_unref (dict);
-
-    return copy;
+    /* Strict match */
+    if ((!a && !b) || (a && b && g_strcmp0 (a, b) == 0))
+        return TRUE;
+    /* Additional loose match, consider NULL and EMPTY string equal */
+    if (flags & MM_BEARER_PROPERTIES_CMP_FLAGS_LOOSE) {
+        if ((!a && !b[0]) || (!b && !a[0]))
+            return TRUE;
+    }
+    return FALSE;
 }
 
-/*****************************************************************************/
-
-gboolean
-mm_bearer_properties_cmp (MMBearerProperties *a,
-                          MMBearerProperties *b)
+static gboolean
+cmp_ip_type (MMBearerIpFamily           a,
+             MMBearerIpFamily           b,
+             MMBearerPropertiesCmpFlags flags)
 {
-    return ((!g_strcmp0 (a->priv->apn, b->priv->apn)) &&
-            (a->priv->ip_type == b->priv->ip_type) &&
-            (!g_strcmp0 (a->priv->number, b->priv->number)) &&
-            (a->priv->allowed_auth == b->priv->allowed_auth) &&
-            (!g_strcmp0 (a->priv->user, b->priv->user)) &&
-            (!g_strcmp0 (a->priv->password, b->priv->password)) &&
-            (a->priv->allow_roaming == b->priv->allow_roaming) &&
-            (a->priv->allow_roaming_set == b->priv->allow_roaming_set) &&
-            (a->priv->rm_protocol == b->priv->rm_protocol));
+    /* Strict match */
+    if (a == b)
+        return TRUE;
+    /* Additional loose match NONE == IPV4 */
+    if (flags & MM_BEARER_PROPERTIES_CMP_FLAGS_LOOSE) {
+        if ((a == MM_BEARER_IP_FAMILY_NONE && b == MM_BEARER_IP_FAMILY_IPV4) ||
+            (b == MM_BEARER_IP_FAMILY_NONE && a == MM_BEARER_IP_FAMILY_IPV4))
+            return TRUE;
+    }
+    return FALSE;
+}
+
+static gboolean
+cmp_allowed_auth (MMBearerAllowedAuth        a,
+                  MMBearerAllowedAuth        b,
+                  MMBearerPropertiesCmpFlags flags)
+{
+    /* Strict match */
+    if (a == b)
+        return TRUE;
+    /* Additional loose match UNKNOWN == NONE */
+    if (flags & MM_BEARER_PROPERTIES_CMP_FLAGS_LOOSE) {
+        if ((a == MM_BEARER_ALLOWED_AUTH_UNKNOWN && b == MM_BEARER_ALLOWED_AUTH_NONE) ||
+            (b == MM_BEARER_ALLOWED_AUTH_UNKNOWN && a == MM_BEARER_ALLOWED_AUTH_NONE))
+            return TRUE;
+    }
+    return FALSE;
+}
+
+static gboolean
+cmp_allow_roaming (gboolean                   a,
+                   gboolean                   a_set,
+                   gboolean                   b,
+                   gboolean                   b_set,
+                   MMBearerPropertiesCmpFlags flags)
+{
+    /* Strict match */
+    if (a == b && a_set == b_set)
+        return TRUE;
+    /* Additional loose match UNSET ==  */
+    if (flags & MM_BEARER_PROPERTIES_CMP_FLAGS_LOOSE) {
+        if ((a == MM_BEARER_ALLOWED_AUTH_UNKNOWN && b == MM_BEARER_ALLOWED_AUTH_NONE) ||
+            (b == MM_BEARER_ALLOWED_AUTH_UNKNOWN && a == MM_BEARER_ALLOWED_AUTH_NONE))
+            return TRUE;
+    }
+    return FALSE;
+}
+
+/**
+ * mm_bearer_properties_cmp: (skip)
+ */
+gboolean
+mm_bearer_properties_cmp (MMBearerProperties         *a,
+                          MMBearerProperties         *b,
+                          MMBearerPropertiesCmpFlags  flags)
+{
+    if (!cmp_str (a->priv->apn, b->priv->apn, flags))
+        return FALSE;
+    if (!cmp_ip_type (a->priv->ip_type, b->priv->ip_type, flags))
+        return FALSE;
+    if (!cmp_allowed_auth (a->priv->allowed_auth, a->priv->allowed_auth, flags))
+        return FALSE;
+    if (!cmp_str (a->priv->user, b->priv->user, flags))
+        return FALSE;
+    if (!(flags & MM_BEARER_PROPERTIES_CMP_FLAGS_NO_PASSWORD) && !cmp_str (a->priv->password, b->priv->password, flags))
+        return FALSE;
+    if (!(flags & MM_BEARER_PROPERTIES_CMP_FLAGS_NO_ALLOW_ROAMING)) {
+        if (a->priv->allow_roaming != b->priv->allow_roaming)
+            return FALSE;
+        if (a->priv->allow_roaming_set != b->priv->allow_roaming_set)
+            return FALSE;
+    }
+    if (a->priv->rm_protocol != b->priv->rm_protocol)
+        return FALSE;
+    return TRUE;
 }
 
 /*****************************************************************************/
@@ -655,6 +777,8 @@ mm_bearer_properties_cmp (MMBearerProperties *a,
  * Creates a new empty #MMBearerProperties.
  *
  * Returns: (transfer full): a #MMBearerProperties. The returned value should be freed with g_object_unref().
+ *
+ * Since: 1.0
  */
 MMBearerProperties *
 mm_bearer_properties_new (void)
@@ -674,13 +798,7 @@ mm_bearer_properties_init (MMBearerProperties *self)
     self->priv->allow_roaming = TRUE;
     self->priv->rm_protocol = MM_MODEM_CDMA_RM_PROTOCOL_UNKNOWN;
     self->priv->allowed_auth = MM_BEARER_ALLOWED_AUTH_UNKNOWN;
-
-    /* At some point in the future, this default should probably be changed
-     * to IPV4V6. However, presently support for this PDP type is rare. An
-     * even better approach would likely be to query which PDP types the
-     * modem supports (using AT+CGDCONT=?), and set the default accordingly
-     */
-    self->priv->ip_type = MM_BEARER_IP_FAMILY_IPV4;
+    self->priv->ip_type = MM_BEARER_IP_FAMILY_NONE;
 }
 
 static void
@@ -691,7 +809,6 @@ finalize (GObject *object)
     g_free (self->priv->apn);
     g_free (self->priv->user);
     g_free (self->priv->password);
-    g_free (self->priv->number);
 
     G_OBJECT_CLASS (mm_bearer_properties_parent_class)->finalize (object);
 }
