@@ -509,23 +509,6 @@ test_scfg_response_2g (void)
     g_array_unref (expected_bands);
 }
 
-static void
-test_scfg_response_2g_ucs2 (void)
-{
-    GArray *expected_bands;
-    MMModemBand single;
-    const gchar *response =
-        "^SCFG: \"Radio/Band\",\"0031\",\"0031\"\r\n"
-        "\r\n";
-
-    expected_bands = g_array_sized_new (FALSE, FALSE, sizeof (MMModemBand), 9);
-    single = MM_MODEM_BAND_EGSM,  g_array_append_val (expected_bands, single);
-
-    common_test_scfg_response (response, MM_MODEM_CHARSET_UCS2, expected_bands, MM_CINTERION_MODEM_FAMILY_DEFAULT, MM_CINTERION_RADIO_BAND_FORMAT_SINGLE);
-
-    g_array_unref (expected_bands);
-}
-
 static void
 test_scfg_response_3g (void)
 {
@@ -1613,6 +1596,120 @@ test_smoni_response_to_signal (void)
     }
 }
 
+/*****************************************************************************/
+/* Test ^SCFG="MEopMode/Prov/Cfg" responses */
+
+typedef struct {
+    const gchar            *str;
+    MMCinterionModemFamily  modem_family;
+    gdouble                 expected_cid;
+} ProvcfgResponseTest;
+
+
+static const ProvcfgResponseTest provcfg_response_tests[] = {
+    {
+
+        .str          = "^SCFG: \"MEopMode/Prov/Cfg\",\"vdfde\"",
+        .modem_family = MM_CINTERION_MODEM_FAMILY_DEFAULT,
+        .expected_cid = 1,
+    },
+    {
+
+        .str          = "* ^SCFG: \"MEopMode/Prov/Cfg\",\"attus\"",
+        .modem_family = MM_CINTERION_MODEM_FAMILY_IMT,
+        .expected_cid = 1,
+    },
+    {
+
+        .str          = "* ^SCFG: \"MEopMode/Prov/Cfg\",\"2\"",
+        .modem_family = MM_CINTERION_MODEM_FAMILY_DEFAULT,
+        .expected_cid = 3,
+    },
+    {
+
+        .str          = "* ^SCFG: \"MEopMode/Prov/Cfg\",\"vzwdcus\"",
+        .modem_family = MM_CINTERION_MODEM_FAMILY_DEFAULT,
+        .expected_cid = 3,
+    },
+    {
+
+        .str          = "* ^SCFG: \"MEopMode/Prov/Cfg\",\"tmode\"",
+        .modem_family = MM_CINTERION_MODEM_FAMILY_DEFAULT,
+        .expected_cid = 2,
+    }
+};
+
+static void
+test_provcfg_response (void)
+{
+    guint i;
+
+    for (i = 0; i < G_N_ELEMENTS (provcfg_response_tests); i++) {
+        gint      cid = -1;
+        gboolean  result;
+        GError   *error = NULL;
+
+        result = mm_cinterion_provcfg_response_to_cid (provcfg_response_tests[i].str,
+                                                       provcfg_response_tests[i].modem_family,
+                                                       MM_MODEM_CHARSET_GSM,
+                                                       NULL,
+                                                       &cid,
+                                                       &error);
+        g_assert_no_error (error);
+        g_assert (result);
+        g_assert_cmpuint (cid, ==, provcfg_response_tests[i].expected_cid);
+    }
+}
+
+/*****************************************************************************/
+/* Test ^SGAUTH responses */
+
+static void
+test_sgauth_response (void)
+{
+    gboolean             result;
+    MMBearerAllowedAuth  auth = MM_BEARER_ALLOWED_AUTH_UNKNOWN;
+    gchar               *username = NULL;
+    GError              *error = NULL;
+
+    const gchar *response =
+        "^SGAUTH: 1,2,\"vf\"\r\n"
+        "^SGAUTH: 2,1,\"\"\r\n"
+        "^SGAUTH: 3,0\r\n";
+
+    /* CID 1 */
+    result = mm_cinterion_parse_sgauth_response (response, 1, &auth, &username, &error);
+    g_assert_no_error (error);
+    g_assert (result);
+    g_assert_cmpuint (auth, ==, MM_BEARER_ALLOWED_AUTH_CHAP);
+    g_assert_cmpstr (username, ==, "vf");
+
+    auth = MM_BEARER_ALLOWED_AUTH_UNKNOWN;
+    g_clear_pointer (&username, g_free);
+
+    /* CID 2 */
+    result = mm_cinterion_parse_sgauth_response (response, 2, &auth, &username, &error);
+    g_assert_no_error (error);
+    g_assert (result);
+    g_assert_cmpuint (auth, ==, MM_BEARER_ALLOWED_AUTH_PAP);
+    g_assert_null (username);
+
+    auth = MM_BEARER_ALLOWED_AUTH_UNKNOWN;
+
+    /* CID 3 */
+    result = mm_cinterion_parse_sgauth_response (response, 3, &auth, &username, &error);
+    g_assert_no_error (error);
+    g_assert (result);
+    g_assert_cmpuint (auth, ==, MM_BEARER_ALLOWED_AUTH_NONE);
+    g_assert_null (username);
+
+    auth = MM_BEARER_ALLOWED_AUTH_UNKNOWN;
+
+    /* CID 4 */
+    result = mm_cinterion_parse_sgauth_response (response, 4, &auth, &username, &error);
+    g_assert_error (error, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND);
+    g_assert (!result);
+}
 
 /*****************************************************************************/
 
@@ -1629,7 +1726,6 @@ int main (int argc, char **argv)
     g_test_add_func ("/MM/cinterion/scfg/alas5",              test_scfg_alas5);
     g_test_add_func ("/MM/cinterion/scfg/response/3g",        test_scfg_response_3g);
     g_test_add_func ("/MM/cinterion/scfg/response/2g",        test_scfg_response_2g);
-    g_test_add_func ("/MM/cinterion/scfg/response/2g/ucs2",   test_scfg_response_2g_ucs2);
     g_test_add_func ("/MM/cinterion/scfg/response/pls62/gsm", test_scfg_response_pls62_gsm);
     g_test_add_func ("/MM/cinterion/scfg/response/pls62/ucs2",test_scfg_response_pls62_ucs2);
     g_test_add_func ("/MM/cinterion/scfg/response/alas5",     test_scfg_response_alas5);
@@ -1647,6 +1743,8 @@ int main (int argc, char **argv)
     g_test_add_func ("/MM/cinterion/ctzu/urc/full",           test_ctzu_urc_full);
     g_test_add_func ("/MM/cinterion/smoni/query_response",    test_smoni_response);
     g_test_add_func ("/MM/cinterion/smoni/query_response_to_signal", test_smoni_response_to_signal);
+    g_test_add_func ("/MM/cinterion/scfg/provcfg",            test_provcfg_response);
+    g_test_add_func ("/MM/cinterion/sgauth",                  test_sgauth_response);
 
     return g_test_run ();
 }
