@@ -16,7 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Copyright (C) 2015 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (C) 2015-2019 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #include <string.h>
@@ -34,8 +34,9 @@
 #include "mm-broadband-modem-sierra.h"
 #include "mm-common-sierra.h"
 #include "mm-broadband-modem-telit.h"
+#include "mm-broadband-modem-xmm.h"
 #include "mm-common-telit.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 
 #if defined WITH_QMI
 #include "mm-broadband-modem-qmi.h"
@@ -43,6 +44,8 @@
 
 #if defined WITH_MBIM
 #include "mm-broadband-modem-mbim.h"
+#include "mm-broadband-modem-mbim-xmm.h"
+#include "mm-broadband-modem-foxconn-t77w968.h"
 #endif
 
 #define MAX_PORT_PROBE_TIMEOUTS 3
@@ -167,7 +170,7 @@ response_ready (MMPortSerialAt *port,
     if (error) {
         /* Non-timeout error, jump to next command */
         if (!g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_RESPONSE_TIMEOUT)) {
-            mm_dbg ("(Dell) Error probing AT port: %s", error->message);
+            mm_obj_dbg (probe, "error probing AT port: %s", error->message);
             g_error_free (error);
             custom_init_step_next_command (task);
             return;
@@ -247,8 +250,7 @@ custom_init_step (GTask *task)
 
     /* If cancelled, end without error right away */
     if (g_cancellable_is_cancelled (g_task_get_cancellable (task))) {
-        mm_dbg ("(Dell) no need to keep on running custom init in (%s)",
-                mm_port_get_device (MM_PORT (ctx->port)));
+        mm_obj_dbg (probe, "no need to keep on running custom init: cancelled");
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
@@ -257,10 +259,7 @@ custom_init_step (GTask *task)
 #if defined WITH_QMI
     /* If device has a QMI port, don't run anything else, as we don't care */
     if (mm_port_probe_list_has_qmi_port (mm_device_peek_port_probe_list (mm_port_probe_peek_device (probe)))) {
-        mm_dbg ("(Dell) no need to run custom init in (%s): device has QMI port",
-                mm_port_get_device (MM_PORT (ctx->port)));
-        mm_port_probe_set_result_at (probe, FALSE);
-        mm_port_probe_set_result_qcdm (probe, FALSE);
+        mm_obj_dbg (probe, "no need to run custom init: device has QMI port");
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
@@ -270,10 +269,7 @@ custom_init_step (GTask *task)
 #if defined WITH_MBIM
     /* If device has a MBIM port, don't run anything else, as we don't care */
     if (mm_port_probe_list_has_mbim_port (mm_device_peek_port_probe_list (mm_port_probe_peek_device (probe)))) {
-        mm_dbg ("(Dell) no need to run custom init in (%s): device has MBIM port",
-                mm_port_get_device (MM_PORT (ctx->port)));
-        mm_port_probe_set_result_at (probe, FALSE);
-        mm_port_probe_set_result_qcdm (probe, FALSE);
+        mm_obj_dbg (probe, "no need to run custom init: device has MBIM port");
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
@@ -281,8 +277,7 @@ custom_init_step (GTask *task)
 #endif
 
     if (ctx->timeouts >= MAX_PORT_PROBE_TIMEOUTS) {
-        mm_dbg ("(Dell) couldn't detect real manufacturer in (%s): too many timeouts",
-                mm_port_get_device (MM_PORT (ctx->port)));
+        mm_obj_dbg (probe, "couldn't detect real manufacturer: too many timeouts");
         mm_port_probe_set_result_at (probe, FALSE);
         goto out;
     }
@@ -327,8 +322,7 @@ custom_init_step (GTask *task)
         return;
     }
 
-    mm_dbg ("(Dell) couldn't detect real manufacturer in (%s): all retries consumed",
-            mm_port_get_device (MM_PORT (ctx->port)));
+    mm_obj_dbg (probe, "couldn't detect real manufacturer: all retries consumed");
 out:
     /* Finish custom_init */
     g_task_return_boolean (task, TRUE);
@@ -388,7 +382,7 @@ create_modem (MMPlugin *self,
 
 #if defined WITH_QMI
     if (mm_port_probe_list_has_qmi_port (probes)) {
-        mm_dbg ("QMI-powered Dell-branded modem found...");
+        mm_obj_dbg (self, "QMI-powered Dell-branded modem found...");
         return MM_BASE_MODEM (mm_broadband_modem_qmi_new (uid,
                                                           drivers,
                                                           mm_plugin_get_name (self),
@@ -399,7 +393,26 @@ create_modem (MMPlugin *self,
 
 #if defined WITH_MBIM
     if (mm_port_probe_list_has_mbim_port (probes)) {
-        mm_dbg ("MBIM-powered Dell-branded modem found...");
+        /* Specific implementation for the DW5821e */
+        if (vendor == 0x413c && (product == 0x81d7 || product == 0x81e0)) {
+            mm_obj_dbg (self, "MBIM-powered DW5821e (T77W968) modem found...");
+            return MM_BASE_MODEM (mm_broadband_modem_foxconn_t77w968_new (uid,
+                                                                          drivers,
+                                                                          mm_plugin_get_name (self),
+                                                                          vendor,
+                                                                          product));
+        }
+
+        if (mm_port_probe_list_is_xmm (probes)) {
+            mm_obj_dbg (self, "MBIM-powered XMM-based modem found...");
+            return MM_BASE_MODEM (mm_broadband_modem_mbim_xmm_new (uid,
+                                                                   drivers,
+                                                                   mm_plugin_get_name (self),
+                                                                   vendor,
+                                                                   product));
+        }
+
+        mm_obj_dbg (self, "MBIM-powered Dell-branded modem found...");
         return MM_BASE_MODEM (mm_broadband_modem_mbim_new (uid,
                                                            drivers,
                                                            mm_plugin_get_name (self),
@@ -409,7 +422,7 @@ create_modem (MMPlugin *self,
 #endif
 
     if (port_probe_list_has_manufacturer_port (probes, DELL_MANUFACTURER_NOVATEL)) {
-        mm_dbg ("Novatel-powered Dell-branded modem found...");
+        mm_obj_dbg (self, "Novatel-powered Dell-branded modem found...");
         return MM_BASE_MODEM (mm_broadband_modem_novatel_new (uid,
                                                               drivers,
                                                               mm_plugin_get_name (self),
@@ -418,7 +431,7 @@ create_modem (MMPlugin *self,
     }
 
     if (port_probe_list_has_manufacturer_port (probes, DELL_MANUFACTURER_SIERRA)) {
-        mm_dbg ("Sierra-powered Dell-branded modem found...");
+        mm_obj_dbg (self, "Sierra-powered Dell-branded modem found...");
         return MM_BASE_MODEM (mm_broadband_modem_sierra_new (uid,
                                                              drivers,
                                                              mm_plugin_get_name (self),
@@ -427,7 +440,7 @@ create_modem (MMPlugin *self,
     }
 
     if (port_probe_list_has_manufacturer_port (probes, DELL_MANUFACTURER_TELIT)) {
-        mm_dbg ("Telit-powered Dell-branded modem found...");
+        mm_obj_dbg (self, "Telit-powered Dell-branded modem found...");
         return MM_BASE_MODEM (mm_broadband_modem_telit_new (uid,
                                                             drivers,
                                                             mm_plugin_get_name (self),
@@ -435,7 +448,16 @@ create_modem (MMPlugin *self,
                                                             product));
     }
 
-    mm_dbg ("Dell-branded generic modem found...");
+    if (mm_port_probe_list_is_xmm (probes)) {
+        mm_obj_dbg (self, "XMM-based modem found...");
+        return MM_BASE_MODEM (mm_broadband_modem_xmm_new (uid,
+                                                          drivers,
+                                                          mm_plugin_get_name (self),
+                                                          vendor,
+                                                          product));
+    }
+
+    mm_obj_dbg (self, "Dell-branded generic modem found...");
     return MM_BASE_MODEM (mm_broadband_modem_new (uid,
                                                   drivers,
                                                   mm_plugin_get_name (self),
@@ -446,10 +468,10 @@ create_modem (MMPlugin *self,
 /*****************************************************************************/
 
 static gboolean
-grab_port (MMPlugin *self,
-           MMBaseModem *modem,
-           MMPortProbe *probe,
-           GError **error)
+grab_port (MMPlugin     *self,
+           MMBaseModem  *modem,
+           MMPortProbe  *probe,
+           GError      **error)
 {
     if (MM_IS_BROADBAND_MODEM_SIERRA (modem))
         return mm_common_sierra_grab_port (self, modem, probe, error);
@@ -469,7 +491,7 @@ grab_port (MMPlugin *self,
 G_MODULE_EXPORT MMPlugin *
 mm_plugin_create (void)
 {
-    static const gchar *subsystems[] = { "tty", "net", "usb", NULL };
+    static const gchar *subsystems[] = { "tty", "net", "usbmisc", NULL };
     static const guint16 vendors[] = { 0x413c, 0 };
     static const MMAsyncMethod custom_init = {
         .async  = G_CALLBACK (dell_custom_init),
@@ -478,14 +500,15 @@ mm_plugin_create (void)
 
     return MM_PLUGIN (
         g_object_new (MM_TYPE_PLUGIN_DELL,
-                      MM_PLUGIN_NAME,                  "Dell",
-                      MM_PLUGIN_ALLOWED_SUBSYSTEMS,    subsystems,
-                      MM_PLUGIN_ALLOWED_VENDOR_IDS,    vendors,
-                      MM_PLUGIN_ALLOWED_AT,            TRUE,
-                      MM_PLUGIN_CUSTOM_INIT,           &custom_init,
-                      MM_PLUGIN_ALLOWED_QCDM,          TRUE,
-                      MM_PLUGIN_ALLOWED_QMI,           TRUE,
-                      MM_PLUGIN_ALLOWED_MBIM,          TRUE,
+                      MM_PLUGIN_NAME,               MM_MODULE_NAME,
+                      MM_PLUGIN_ALLOWED_SUBSYSTEMS, subsystems,
+                      MM_PLUGIN_ALLOWED_VENDOR_IDS, vendors,
+                      MM_PLUGIN_ALLOWED_AT,         TRUE,
+                      MM_PLUGIN_CUSTOM_INIT,        &custom_init,
+                      MM_PLUGIN_ALLOWED_QCDM,       TRUE,
+                      MM_PLUGIN_ALLOWED_QMI,        TRUE,
+                      MM_PLUGIN_ALLOWED_MBIM,       TRUE,
+                      MM_PLUGIN_XMM_PROBE,          TRUE,
                       NULL));
 }
 
