@@ -78,7 +78,7 @@ mm_bearer_list_get_count_active (MMBearerList *self)
 
 gboolean
 mm_bearer_list_add_bearer (MMBearerList *self,
-                           MMBearer *bearer,
+                           MMBaseBearer *bearer,
                            GError **error)
 {
     /* Just in case, ensure we don't go off limits */
@@ -105,17 +105,8 @@ mm_bearer_list_delete_bearer (MMBearerList *self,
 {
     GList *l;
 
-    if (!g_str_has_prefix (path, MM_DBUS_BEARER_PREFIX)) {
-        g_set_error (error,
-                     MM_CORE_ERROR,
-                     MM_CORE_ERROR_INVALID_ARGS,
-                     "Cannot delete bearer: invalid path '%s'",
-                     path);
-        return FALSE;
-    }
-
     for (l = self->priv->bearers; l; l = g_list_next (l)) {
-        if (g_str_equal (path, mm_bearer_get_path (MM_BEARER (l->data)))) {
+        if (g_str_equal (path, mm_base_bearer_get_path (MM_BASE_BEARER (l->data)))) {
             g_object_unref (l->data);
             self->priv->bearers = g_list_delete_link (self->priv->bearers, l);
             g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_NUM_BEARERS]);
@@ -131,17 +122,6 @@ mm_bearer_list_delete_bearer (MMBearerList *self,
     return FALSE;
 }
 
-void
-mm_bearer_list_delete_all_bearers (MMBearerList *self)
-{
-    if (!self->priv->bearers)
-        return;
-
-    g_list_free_full (self->priv->bearers, (GDestroyNotify) g_object_unref);
-    self->priv->bearers = NULL;
-    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_NUM_BEARERS]);
-}
-
 GStrv
 mm_bearer_list_get_paths (MMBearerList *self)
 {
@@ -153,7 +133,7 @@ mm_bearer_list_get_paths (MMBearerList *self)
                         1 + g_list_length (self->priv->bearers));
 
     for (i = 0, l = self->priv->bearers; l; l = g_list_next (l))
-        path_list[i++] = g_strdup (mm_bearer_get_path (MM_BEARER (l->data)));
+        path_list[i++] = g_strdup (mm_base_bearer_get_path (MM_BASE_BEARER (l->data)));
 
     return path_list;
 }
@@ -166,14 +146,32 @@ mm_bearer_list_foreach (MMBearerList *self,
     g_list_foreach (self->priv->bearers, (GFunc)func, user_data);
 }
 
-MMBearer *
-mm_bearer_list_find (MMBearerList *self,
-                     MMBearerProperties *properties)
+MMBaseBearer *
+mm_bearer_list_find_by_properties (MMBearerList       *self,
+                                   MMBearerProperties *props)
 {
     GList *l;
 
     for (l = self->priv->bearers; l; l = g_list_next (l)) {
-        if (mm_bearer_properties_cmp (mm_bearer_peek_config (MM_BEARER (l->data)), properties))
+        /* always strict matching when comparing these bearer properties, as they're all
+         * built in the same place */
+        if (mm_bearer_properties_cmp (mm_base_bearer_peek_config (MM_BASE_BEARER (l->data)),
+                                      props,
+                                      MM_BEARER_PROPERTIES_CMP_FLAGS_NONE))
+            return g_object_ref (l->data);
+    }
+
+    return NULL;
+}
+
+MMBaseBearer *
+mm_bearer_list_find_by_path (MMBearerList *self,
+                             const gchar *path)
+{
+    GList *l;
+
+    for (l = self->priv->bearers; l; l = g_list_next (l)) {
+        if (g_str_equal (path, mm_base_bearer_get_path (MM_BASE_BEARER (l->data))))
             return g_object_ref (l->data);
     }
 
@@ -183,19 +181,16 @@ mm_bearer_list_find (MMBearerList *self,
 /*****************************************************************************/
 
 typedef struct {
-    GSimpleAsyncResult *result;
     GList *pending;
-    MMBearer *current;
+    MMBaseBearer *current;
 } DisconnectAllContext;
 
 static void
-disconnect_all_context_complete_and_free (DisconnectAllContext *ctx)
+disconnect_all_context_free (DisconnectAllContext *ctx)
 {
-    g_simple_async_result_complete (ctx->result);
-    g_object_unref (ctx->result);
     if (ctx->current)
         g_object_unref (ctx->current);
-    g_list_free_full (ctx->pending, (GDestroyNotify) g_object_unref);
+    g_list_free_full (ctx->pending, g_object_unref);
     g_free (ctx);
 }
 
@@ -204,46 +199,49 @@ mm_bearer_list_disconnect_all_bearers_finish (MMBearerList *self,
                                               GAsyncResult *res,
                                               GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
-static void disconnect_next_bearer (DisconnectAllContext *ctx);
+static void disconnect_next_bearer (GTask *task);
 
 static void
-disconnect_ready (MMBearer *bearer,
+disconnect_ready (MMBaseBearer *bearer,
                   GAsyncResult *res,
-                  DisconnectAllContext *ctx)
+                  GTask *task)
 {
     GError *error = NULL;
 
-    if (!mm_bearer_disconnect_finish (bearer, res, &error)) {
-        g_simple_async_result_take_error (ctx->result, error);
-        disconnect_all_context_complete_and_free (ctx);
+    if (!mm_base_bearer_disconnect_finish (bearer, res, &error)) {
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
-    disconnect_next_bearer (ctx);
+    disconnect_next_bearer (task);
 }
 
 static void
-disconnect_next_bearer (DisconnectAllContext *ctx)
+disconnect_next_bearer (GTask *task)
 {
+    DisconnectAllContext *ctx;
+
+    ctx = g_task_get_task_data (task);
     if (ctx->current)
         g_clear_object (&ctx->current);
 
     /* No more bearers? all done! */
     if (!ctx->pending) {
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        disconnect_all_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
-    ctx->current = MM_BEARER (ctx->pending->data);
+    ctx->current = MM_BASE_BEARER (ctx->pending->data);
     ctx->pending = g_list_delete_link (ctx->pending, ctx->pending);
 
-    mm_bearer_disconnect (ctx->current,
-                          (GAsyncReadyCallback)disconnect_ready,
-                          ctx);
+    mm_base_bearer_disconnect (ctx->current,
+                               (GAsyncReadyCallback)disconnect_ready,
+                               task);
 }
 
 void
@@ -252,17 +250,20 @@ mm_bearer_list_disconnect_all_bearers (MMBearerList *self,
                                        gpointer user_data)
 {
     DisconnectAllContext *ctx;
+    GTask *task;
 
     ctx = g_new0 (DisconnectAllContext, 1);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_bearer_list_disconnect_all_bearers);
     /* Get a copy of the list */
-    ctx->pending = g_list_copy (self->priv->bearers);
-    g_list_foreach (ctx->pending, (GFunc) g_object_ref, NULL);
+    ctx->pending = g_list_copy_deep (self->priv->bearers,
+                                     (GCopyFunc)g_object_ref,
+                                     NULL);
 
-    disconnect_next_bearer (ctx);
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task,
+                          ctx,
+                          (GDestroyNotify)disconnect_all_context_free);
+
+    disconnect_next_bearer (task);
 }
 
 /*****************************************************************************/
@@ -271,10 +272,6 @@ MMBearerList *
 mm_bearer_list_new (guint max_bearers,
                     guint max_active_bearers)
 {
-    mm_dbg ("Creating bearer list (max: %u, max active: %u)",
-            max_bearers,
-            max_active_bearers);
-
     /* Create the object */
     return g_object_new  (MM_TYPE_BEARER_LIST,
                           MM_BEARER_LIST_MAX_BEARERS, max_bearers,
@@ -334,7 +331,7 @@ static void
 mm_bearer_list_init (MMBearerList *self)
 {
     /* Initialize private data */
-    self->priv = G_TYPE_INSTANCE_GET_PRIVATE ((self),
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                               MM_TYPE_BEARER_LIST,
                                               MMBearerListPrivate);
 }
@@ -344,7 +341,10 @@ dispose (GObject *object)
 {
     MMBearerList *self = MM_BEARER_LIST (object);
 
-    mm_bearer_list_delete_all_bearers (self);
+    if (self->priv->bearers) {
+        g_list_free_full (self->priv->bearers, g_object_unref);
+        self->priv->bearers = NULL;
+    }
 
     G_OBJECT_CLASS (mm_bearer_list_parent_class)->dispose (object);
 }
