@@ -35,6 +35,7 @@
 #define MM_IFACE_MODEM_DBUS_SKELETON           "iface-modem-dbus-skeleton"
 #define MM_IFACE_MODEM_STATE                   "iface-modem-state"
 #define MM_IFACE_MODEM_SIM                     "iface-modem-sim"
+#define MM_IFACE_MODEM_SIM_SLOTS               "iface-modem-sim-slots"
 #define MM_IFACE_MODEM_BEARER_LIST             "iface-modem-bearer-list"
 #define MM_IFACE_MODEM_SIM_HOT_SWAP_SUPPORTED  "iface-modem-sim-hot-swap-supported"
 #define MM_IFACE_MODEM_SIM_HOT_SWAP_CONFIGURED "iface-modem-sim-hot-swap-configured"
@@ -289,6 +290,7 @@ struct _MMIfaceModem {
      * Useful for when the modem changes power states since we might
      * not get the relevant notifications from the modem. */
     void (*check_for_sim_swap) (MMIfaceModem *self,
+                                const gchar *iccid,
                                 GAsyncReadyCallback callback,
                                 gpointer user_data);
     gboolean (*check_for_sim_swap_finish) (MMIfaceModem *self,
@@ -343,6 +345,25 @@ struct _MMIfaceModem {
                                       GAsyncResult *res,
                                       GError **error);
 
+    /* Create SIMs in all SIM slots */
+    void     (* load_sim_slots)        (MMIfaceModem         *self,
+                                        GAsyncReadyCallback   callback,
+                                        gpointer              user_data);
+    gboolean (* load_sim_slots_finish) (MMIfaceModem         *self,
+                                        GAsyncResult         *res,
+                                        GPtrArray           **sim_slots,
+                                        guint                *primary_sim_slot,
+                                        GError              **error);
+
+    /* Set primary SIM slot */
+    void     (* set_primary_sim_slot)        (MMIfaceModem         *self,
+                                              guint                 sim_slot,
+                                              GAsyncReadyCallback   callback,
+                                              gpointer              user_data);
+    gboolean (* set_primary_sim_slot_finish) (MMIfaceModem         *self,
+                                              GAsyncResult         *res,
+                                              GError              **error);
+
     /* Create bearer */
     void (*create_bearer) (MMIfaceModem *self,
                            MMBearerProperties *properties,
@@ -382,6 +403,7 @@ struct _MMIfaceModem {
 };
 
 GType mm_iface_modem_get_type (void);
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMIfaceModem, g_object_unref)
 
 /* Helpers to query access technologies */
 MMModemAccessTechnology mm_iface_modem_get_access_technologies (MMIfaceModem *self);
@@ -391,7 +413,7 @@ MMModemCapability mm_iface_modem_get_current_capabilities (MMIfaceModem *self);
 gboolean          mm_iface_modem_is_3gpp                  (MMIfaceModem *self);
 gboolean          mm_iface_modem_is_3gpp_only             (MMIfaceModem *self);
 gboolean          mm_iface_modem_is_3gpp_lte              (MMIfaceModem *self);
-gboolean          mm_iface_modem_is_3gpp_lte_only         (MMIfaceModem *self);
+gboolean          mm_iface_modem_is_3gpp_5gnr             (MMIfaceModem *self);
 gboolean          mm_iface_modem_is_cdma                  (MMIfaceModem *self);
 gboolean          mm_iface_modem_is_cdma_only             (MMIfaceModem *self);
 
@@ -402,6 +424,8 @@ gboolean mm_iface_modem_is_3g      (MMIfaceModem *self);
 gboolean mm_iface_modem_is_3g_only (MMIfaceModem *self);
 gboolean mm_iface_modem_is_4g      (MMIfaceModem *self);
 gboolean mm_iface_modem_is_4g_only (MMIfaceModem *self);
+gboolean mm_iface_modem_is_5g      (MMIfaceModem *self);
+gboolean mm_iface_modem_is_5g_only (MMIfaceModem *self);
 
 /* Helpers to query properties */
 const gchar *mm_iface_modem_get_model          (MMIfaceModem  *self);
@@ -460,7 +484,8 @@ MMModemLock mm_iface_modem_update_lock_info_finish (MMIfaceModem *self,
                                                     GAsyncResult *res,
                                                     GError **error);
 
-MMUnlockRetries *mm_iface_modem_get_unlock_retries (MMIfaceModem *self);
+MMModemLock      mm_iface_modem_get_unlock_required (MMIfaceModem *self);
+MMUnlockRetries *mm_iface_modem_get_unlock_retries  (MMIfaceModem *self);
 
 void mm_iface_modem_update_unlock_retries (MMIfaceModem *self,
                                            MMUnlockRetries *unlock_retries);
@@ -543,4 +568,14 @@ MMModemState mm_iface_modem_wait_for_final_state_finish (MMIfaceModem *self,
 void mm_iface_modem_bind_simple_status (MMIfaceModem *self,
                                         MMSimpleStatus *status);
 
+/* Check if the SIM or eSIM profile has changed */
+void     mm_iface_modem_check_for_sim_swap        (MMIfaceModem *self,
+                                                   guint slot_index,
+                                                   const gchar *iccid,
+                                                   GAsyncReadyCallback callback,
+                                                   gpointer user_data);
+gboolean mm_iface_modem_check_for_sim_swap_finish (MMIfaceModem *self,
+                                                   GAsyncResult *res,
+                                                   GError **error);
+
 #endif /* MM_IFACE_MODEM_H */
