@@ -39,13 +39,10 @@
 /* Common utilities */
 /*****************************************************************************/
 
-#define MM_MODEM_CAPABILITY_3GPP_LTE    \
-    (MM_MODEM_CAPABILITY_LTE |          \
-     MM_MODEM_CAPABILITY_LTE_ADVANCED)
-
 #define MM_MODEM_CAPABILITY_3GPP        \
     (MM_MODEM_CAPABILITY_GSM_UMTS |     \
-     MM_MODEM_CAPABILITY_3GPP_LTE)
+     MM_MODEM_CAPABILITY_LTE |          \
+     MM_MODEM_CAPABILITY_5GNR)
 
 gchar       *mm_strip_quotes (gchar *str);
 const gchar *mm_strip_tag    (const gchar *str,
@@ -59,8 +56,9 @@ GArray *mm_parse_uint_list (const gchar  *str,
 guint mm_count_bits_set (gulong number);
 guint mm_find_bit_set   (gulong number);
 
-gchar *mm_create_device_identifier (guint vid,
-                                    guint pid,
+gchar *mm_create_device_identifier (guint        vid,
+                                    guint        pid,
+                                    gpointer     log_object,
                                     const gchar *ati,
                                     const gchar *ati1,
                                     const gchar *gsn,
@@ -83,17 +81,34 @@ gchar *mm_new_iso8601_time (guint year,
                             gint offset_minutes);
 
 GArray *mm_filter_supported_modes (const GArray *all,
-                                   const GArray *supported_combinations);
+                                   const GArray *supported_combinations,
+                                   gpointer      log_object);
 
-GArray *mm_filter_supported_capabilities (MMModemCapability all,
-                                          const GArray *supported_combinations);
+gchar *mm_bcd_to_string (const guint8 *bcd,
+                         gsize bcd_len,
+                         gboolean low_nybble_first);
 
 /*****************************************************************************/
 /* VOICE specific helpers and utilities */
 /*****************************************************************************/
+
 GRegex *mm_voice_ring_regex_get  (void);
 GRegex *mm_voice_cring_regex_get (void);
 GRegex *mm_voice_clip_regex_get  (void);
+GRegex *mm_voice_ccwa_regex_get  (void);
+
+/* +CLCC response parser */
+typedef struct {
+    guint            index;
+    MMCallDirection  direction;
+    MMCallState      state;
+    gchar           *number; /* optional */
+} MMCallInfo;
+gboolean mm_3gpp_parse_clcc_response (const gchar  *str,
+                                      gpointer      log_object,
+                                      GList       **out_list,
+                                      GError      **error);
+void     mm_3gpp_call_info_list_free (GList        *call_info_list);
 
 /*****************************************************************************/
 /* SERIAL specific helpers and utilities */
@@ -110,8 +125,12 @@ typedef enum { /*< underscore_name=mm_flow_control >*/
 } MMFlowControl;
 
 MMFlowControl mm_parse_ifc_test_response (const gchar  *response,
+                                          gpointer      log_object,
                                           GError      **error);
 
+MMFlowControl mm_flow_control_from_string (const gchar  *str,
+                                           GError      **error);
+
 /*****************************************************************************/
 /* 3GPP specific helpers and utilities */
 /*****************************************************************************/
@@ -120,6 +139,7 @@ MMFlowControl mm_parse_ifc_test_response (const gchar  *response,
 GPtrArray *mm_3gpp_creg_regex_get     (gboolean solicited);
 void       mm_3gpp_creg_regex_destroy (GPtrArray *array);
 GRegex    *mm_3gpp_ciev_regex_get (void);
+GRegex    *mm_3gpp_cgev_regex_get (void);
 GRegex    *mm_3gpp_cusd_regex_get (void);
 GRegex    *mm_3gpp_cmti_regex_get (void);
 GRegex    *mm_3gpp_cds_regex_get (void);
@@ -137,8 +157,10 @@ typedef struct {
     MMModemAccessTechnology access_tech;
 } MM3gppNetworkInfo;
 void mm_3gpp_network_info_list_free (GList *info_list);
-GList *mm_3gpp_parse_cops_test_response (const gchar *reply,
-                                         GError **error);
+GList *mm_3gpp_parse_cops_test_response (const gchar     *reply,
+                                         MMModemCharset   cur_charset,
+                                         gpointer         log_object,
+                                         GError         **error);
 
 /* AT+COPS? (current operator) response parser */
 gboolean mm_3gpp_parse_cops_read_response (const gchar              *response,
@@ -159,8 +181,9 @@ typedef struct {
     MMBearerIpFamily pdp_type;
 } MM3gppPdpContextFormat;
 void mm_3gpp_pdp_context_format_list_free (GList *pdp_format_list);
-GList *mm_3gpp_parse_cgdcont_test_response (const gchar *reply,
-                                            GError **error);
+GList *mm_3gpp_parse_cgdcont_test_response (const gchar  *reply,
+                                            gpointer      log_object,
+                                            GError      **error);
 
 /* AT+CGDCONT? (PDP context query) response parser */
 typedef struct {
@@ -172,24 +195,37 @@ void mm_3gpp_pdp_context_list_free (GList *pdp_list);
 GList *mm_3gpp_parse_cgdcont_read_response (const gchar *reply,
                                             GError **error);
 
+/* Select best CID to use during connection */
+guint mm_3gpp_select_best_cid (const gchar      *apn,
+                               MMBearerIpFamily  ip_family,
+                               GList            *context_list,
+                               GList            *context_format_list,
+                               gpointer          log_object,
+                               gboolean         *out_cid_reused,
+                               gboolean         *out_cid_overwritten);
+
 /* AT+CGACT? (active PDP context query) response parser */
 typedef struct {
     guint cid;
     gboolean active;
 } MM3gppPdpContextActive;
 void mm_3gpp_pdp_context_active_list_free (GList *pdp_active_list);
+gint mm_3gpp_pdp_context_active_cmp (MM3gppPdpContextActive *a,
+                                     MM3gppPdpContextActive *b);
 GList *mm_3gpp_parse_cgact_read_response (const gchar *reply,
                                           GError **error);
 
 /* CREG/CGREG response/unsolicited message parser */
-gboolean mm_3gpp_parse_creg_response (GMatchInfo *info,
-                                      MMModem3gppRegistrationState *out_reg_state,
-                                      gulong *out_lac,
-                                      gulong *out_ci,
-                                      MMModemAccessTechnology *out_act,
-                                      gboolean *out_cgreg,
-                                      gboolean *out_cereg,
-                                      GError **error);
+gboolean mm_3gpp_parse_creg_response (GMatchInfo                    *info,
+                                      gpointer                       log_object,
+                                      MMModem3gppRegistrationState  *out_reg_state,
+                                      gulong                        *out_lac,
+                                      gulong                        *out_ci,
+                                      MMModemAccessTechnology       *out_act,
+                                      gboolean                      *out_cgreg,
+                                      gboolean                      *out_cereg,
+                                      gboolean                      *out_c5greg,
+                                      GError                       **error);
 
 /* AT+CMGF=? (SMS message format) response parser */
 gboolean mm_3gpp_parse_cmgf_test_response (const gchar *reply,
@@ -198,10 +234,11 @@ gboolean mm_3gpp_parse_cmgf_test_response (const gchar *reply,
                                            GError **error);
 
 /* AT+CPMS=? (Preferred SMS storage) response parser */
-gboolean mm_3gpp_parse_cpms_test_response (const gchar *reply,
-                                           GArray **mem1,
-                                           GArray **mem2,
-                                           GArray **mem3);
+gboolean mm_3gpp_parse_cpms_test_response (const gchar  *reply,
+                                           GArray      **mem1,
+                                           GArray      **mem2,
+                                           GArray      **mem3,
+                                           GError      **error);
 
 /* AT+CPMS? (Current SMS storage) response parser */
 gboolean mm_3gpp_parse_cpms_query_response (const gchar *reply,
@@ -248,6 +285,7 @@ typedef enum { /*< underscore_name=mm_3gpp_cmer_ind >*/
 gchar    *mm_3gpp_build_cmer_set_request   (MM3gppCmerMode   mode,
                                             MM3gppCmerInd    ind);
 gboolean  mm_3gpp_parse_cmer_test_response (const gchar     *reply,
+                                            gpointer         log_object,
                                             MM3gppCmerMode  *supported_modes,
                                             MM3gppCmerInd   *supported_inds,
                                             GError         **error);
@@ -265,6 +303,47 @@ gint         mm_3gpp_cind_response_get_max       (MM3gppCindResponse *r);
 GByteArray *mm_3gpp_parse_cind_read_response (const gchar *reply,
                                               GError **error);
 
+/* +CGEV indication parser */
+typedef enum {
+    MM_3GPP_CGEV_UNKNOWN,
+    MM_3GPP_CGEV_NW_DETACH,
+    MM_3GPP_CGEV_ME_DETACH,
+    MM_3GPP_CGEV_NW_CLASS,
+    MM_3GPP_CGEV_ME_CLASS,
+    MM_3GPP_CGEV_NW_ACT_PRIMARY,
+    MM_3GPP_CGEV_ME_ACT_PRIMARY,
+    MM_3GPP_CGEV_NW_ACT_SECONDARY,
+    MM_3GPP_CGEV_ME_ACT_SECONDARY,
+    MM_3GPP_CGEV_NW_DEACT_PRIMARY,
+    MM_3GPP_CGEV_ME_DEACT_PRIMARY,
+    MM_3GPP_CGEV_NW_DEACT_SECONDARY,
+    MM_3GPP_CGEV_ME_DEACT_SECONDARY,
+    MM_3GPP_CGEV_NW_DEACT_PDP,
+    MM_3GPP_CGEV_ME_DEACT_PDP,
+    MM_3GPP_CGEV_NW_MODIFY,
+    MM_3GPP_CGEV_ME_MODIFY,
+    MM_3GPP_CGEV_REJECT,
+    MM_3GPP_CGEV_NW_REACT,
+} MM3gppCgev;
+
+MM3gppCgev mm_3gpp_parse_cgev_indication_action    (const gchar *str);
+gboolean   mm_3gpp_parse_cgev_indication_pdp       (const gchar  *str,
+                                                    MM3gppCgev    type,
+                                                    gchar       **out_pdp_type,
+                                                    gchar       **out_pdp_addr,
+                                                    guint        *out_cid,
+                                                    GError      **error);
+gboolean   mm_3gpp_parse_cgev_indication_primary   (const gchar  *str,
+                                                    MM3gppCgev    type,
+                                                    guint        *out_cid,
+                                                    GError      **error);
+gboolean   mm_3gpp_parse_cgev_indication_secondary (const gchar  *str,
+                                                    MM3gppCgev    type,
+                                                    guint        *out_p_cid,
+                                                    guint        *out_cid,
+                                                    guint        *out_event_type,
+                                                    GError      **error);
+
 /* AT+CMGL=4 (list sms parts) response parser */
 typedef struct {
     gint index;
@@ -323,6 +402,7 @@ gboolean mm_3gpp_parse_cesq_response (const gchar  *response,
                                       GError      **error);
 
 gboolean mm_3gpp_cesq_response_to_signal_info (const gchar  *response,
+                                               gpointer      log_object,
                                                MMSignal    **out_gsm,
                                                MMSignal    **out_umts,
                                                MMSignal    **out_lte,
@@ -334,10 +414,17 @@ gboolean  mm_3gpp_parse_cemode_query_response (const gchar                    *r
                                                MMModem3gppEpsUeModeOperation  *out_mode,
                                                GError                        **error);
 
+/* CCWA service query response parser */
+gboolean mm_3gpp_parse_ccwa_service_query_response (const gchar  *response,
+                                                    gpointer      log_object,
+                                                    gboolean     *status,
+                                                    GError      **error);
+
+
 /* Additional 3GPP-specific helpers */
 
-MMModem3gppFacility mm_3gpp_acronym_to_facility (const gchar *str);
-gchar *mm_3gpp_facility_to_acronym (MMModem3gppFacility facility);
+MMModem3gppFacility  mm_3gpp_acronym_to_facility (const gchar         *str);
+const gchar         *mm_3gpp_facility_to_acronym (MMModem3gppFacility  facility);
 
 MMModemAccessTechnology mm_string_to_access_tech (const gchar *string);
 
@@ -354,6 +441,28 @@ MMBearerIpFamily  mm_3gpp_get_ip_family_from_pdp_type (const gchar *pdp_type);
 
 char *mm_3gpp_parse_iccid (const char *raw_iccid, GError **error);
 
+
+gboolean mm_3gpp_rscp_level_to_rscp   (guint     rscp_level,
+                                       gpointer  log_object,
+                                       gdouble  *out_rscp);
+gboolean mm_3gpp_rxlev_to_rssi        (guint     rxlev,
+                                       gpointer  log_object,
+                                       gdouble  *out_rssi);
+gboolean mm_3gpp_ecn0_level_to_ecio   (guint     ecn0_level,
+                                       gpointer  log_object,
+                                       gdouble  *out_ecio);
+gboolean mm_3gpp_rsrq_level_to_rsrq   (guint     rsrq_level,
+                                       gpointer  log_object,
+                                       gdouble  *out_rsrq);
+gboolean mm_3gpp_rsrp_level_to_rsrp   (guint     rsrp_level,
+                                       gpointer  log_object,
+                                       gdouble  *out_rsrp);
+gboolean mm_3gpp_rssnr_level_to_rssnr (gint      rssnr_level,
+                                       gpointer  log_object,
+                                       gdouble  *out_rssnr);
+
+GStrv mm_3gpp_parse_emergency_numbers (const char *raw, GError **error);
+
 /*****************************************************************************/
 /* CDMA specific helpers and utilities */
 /*****************************************************************************/
@@ -404,4 +513,16 @@ gboolean mm_parse_cclk_response (const gchar *response,
 gint mm_parse_csim_response (const gchar *response,
                                    GError **error);
 
+gboolean mm_parse_supl_address (const gchar  *supl,
+                                gchar       **out_fqdn,
+                                guint32      *out_ip,
+                                guint16      *out_port,
+                                GError      **error);
+
+/*****************************************************************************/
+
+/* Useful when clamp-ing an unsigned integer with implicit low limit set to 0,
+ * and in order to avoid -Wtype-limits warnings. */
+#define MM_CLAMP_HIGH(x, high) (((x) > (high)) ? (high) : (x))
+
 #endif  /* MM_MODEM_HELPERS_H */
